package com.lindosoft.android.tongue;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.PowerManager;
import android.view.GestureDetector;
import android.view.GestureDetector.SimpleOnGestureListener;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.VideoView;
import java.io.File;

/** Tongues base. */
public abstract class TonguesBase extends Activity implements MediaPlayer.OnCompletionListener, MediaPlayer.OnErrorListener {
  protected boolean first = true;
  private long touch = 0L;
  private Object lock = new Object();
  private GestureDetector gestureDetector = new GestureDetector(new SwipeOnGestureListener());
  private int screenOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
  private PowerManager.WakeLock wl = null;

  @Override
  public void onCompletion(MediaPlayer mediaPlayer) {
    stopVideo(true);
    startNext();
  }

  @Override
  public boolean onError(MediaPlayer mp, int what, int extra) {
    stopVideo(false);
    startNext();
    return true;
  }
  
  
  /** Override this. */
  protected abstract void doFirst();

  /** Override this. */
  protected abstract void doSecond();

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    requestWindowFeature(Window.FEATURE_NO_TITLE);
    getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
    setContentView(R.layout.tongues);
    PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
    wl = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ON_AFTER_RELEASE, "lindotongue");
    wl.acquire();
    View layout = findViewById(R.id.layout_tongues);
    layout.setOnTouchListener(new OnTouchListener() {
      public boolean onTouch(View v, MotionEvent event) {
        return gestureDetector.onTouchEvent(event);
      }
    }); 
    setRequestedOrientation(screenOrientation);
    doFirst();
  }

  @Override
  protected void onDestroy() {
    stopNext();
    stopVideo(false);
    wl.release();
    super.onDestroy();
  }

  /** Next doFirst or doSecond. */
  protected void next() {
    if (first) {
      doFirst();
    } else {
      doSecond();
    }
  }

  /** Rotate screen by 90 degrees. */
  protected void rotate() {
    switch(screenOrientation) {
      case ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE:
        screenOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT;
        break;
      case ActivityInfo.SCREEN_ORIENTATION_PORTRAIT:
        screenOrientation = ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE;
        break;
      case ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE:
        screenOrientation = ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT;
        break;
      case ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT:
        screenOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
        break;
    }
    setRequestedOrientation(screenOrientation);
  }

  /** Set name. */
  protected void setName(String name) {
    int[] viewIds = {R.id.text_top, R.id.text_left, R.id.text_bottom, R.id.text_right};
    for (int viewId : viewIds) {
      TextView textView = (TextView) findViewById(viewId);
      textView.setText(0 == name.length() ? "?" : name);
    }
  }

  /** Start video. */
  protected void startVideo(String video, boolean visibility) {
    stopNext();
    try {
      VideoView videoView = (VideoView) findViewById(R.id.video);
      videoView.setVisibility(View.INVISIBLE);
      String videoPath = new File(FolderModel.folder, video).getAbsolutePath();
      videoView.setOnCompletionListener(this);
      videoView.setOnErrorListener(this);
      videoView.setVideoPath(videoPath);
      View black = (View) findViewById(R.id.black);
      black.setAlpha(visibility ? 0.0f : 1.0f);
      videoView.setVisibility(View.VISIBLE);
      videoView.start();
    } catch (Exception ex) {
      stopVideo(false);
    }
  }

  /** Start next. */
  protected void startNext() {
    stopNext();
    if (!MainModel.auto) {
      return;
    }
    final long touchTemp = touch;
    new Thread(new Runnable() {
      public void run() {
        synchronized(lock) {
          try {
            lock.wait(MainModel.pause);
          } catch (Exception ignore) {
          }
        }
        if (touchTemp == touch) {
          runOnUiThread(new Runnable() {
            public void run() {
              next();
            }
          });
        }
      }
    }).start();
  }

  /** Stop video. */
  protected void stopVideo(boolean visibility) {
    try {
      VideoView videoView = (VideoView) findViewById(R.id.video);
      videoView.setVisibility(visibility ? View.VISIBLE : View.INVISIBLE);
      videoView.stopPlayback();
    } catch (Exception ex) {
    }
  }

  /** Stop next. */
  protected void stopNext() {
    touch = System.currentTimeMillis();
    synchronized(lock) {
      lock.notifyAll();
    }
  }

  private class SwipeOnGestureListener extends SimpleOnGestureListener {
    @Override
    public boolean onDown(MotionEvent e) {
      return true;
    }

    @Override
    public boolean onSingleTapConfirmed (MotionEvent e) {
      rotate();
      return true;
    }

    @Override
    public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
      float diffX = e2.getX() - e1.getX();
      float diffY = e2.getY() - e1.getY();
      if ((diffX < -MainModel.swipeThreshold && Math.abs(velocityX) > MainModel.swipeThreshold / 2)
          || (diffY > MainModel.swipeThreshold && Math.abs(velocityY) > MainModel.swipeThreshold / 2)) {
        stopNext();
        stopVideo(false);
        next();
      } else if ((diffX > MainModel.swipeThreshold && Math.abs(velocityX) > MainModel.swipeThreshold / 2)
          || (diffY < -MainModel.swipeThreshold && Math.abs(velocityY) > MainModel.swipeThreshold / 2)) {
        stopNext();
        stopVideo(false);
        TongueModel.previousTongue();
        TongueModel.previousTongue();
        doFirst();
      }
      return true;
    }
  }
}